==> conc-2.0.1.tar.gz: conc-2.0.1/client/config-logs.c <==
#include <string.h>
#include "config-logs.h"
#include "spec-log.h"
#include "error.h"
#include "client-message.h"

typedef struct _ConfigLogDialog {
    Conc *conc;
    gchar *name; /* NULL if adding new log */
    GladeXML *xml; /* NULL if removing log */
    GtkTreeView *treeview;
    GtkListStore *members;
} ConfigLogDialog;

enum {
    COLUMN_MEMBER,
    COLUMN_LAST
};

/*
 * Glade widget names:
 */
#define CONFIG_LOG_DIALOG			"config log dialog"
#define CONFIG_LOG_DIALOG_NAME			"name"
#define CONFIG_LOG_DIALOG_SERVER		"server"
#define CONFIG_LOG_DIALOG_FILE			"file"
#define CONFIG_LOG_DIALOG_CONSOLES		"consoles"
#define CONFIG_LOG_DIALOG_CONSOLE		"console"
#define CONFIG_LOG_DIALOG_ADD			"add"
#define CONFIG_LOG_DIALOG_REMOVE		"remove"

/*
 * Glade handler names:
 */
#define CONFIG_LOG_DIALOG_RESPONSE		"response"
#define CONFIG_LOG_DIALOG_CONSOLES_CHANGED	"consoles changed"
#define CONFIG_LOG_DIALOG_CONSOLE_CHANGED	"console changed"
#define CONFIG_LOG_DIALOG_ADD_CONSOLE		"console add"
#define CONFIG_LOG_DIALOG_REMOVE_CONSOLE	"console remove"

static void
config_logs_dialog_destroy(ConfigLogDialog *cld) {
    if (cld->xml) {
	gtk_widget_destroy(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG));
    }
    if (cld->name) {
	g_free(cld->name);
    }
    if (cld->members) {
	g_object_unref(cld->members);
    }
    g_free(cld);
}

static void
config_logs_done(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    ConfigLogDialog *cld = data;
    GError *e = message_arg_error_check(arg);
    gchar *failed_title = _("Failed to update log");
    gchar *failed_text = _("Could not modify log (%s)");

    if (cld->xml == NULL) {
	failed_title = _("Failed to remove log");
	failed_text = _("Could not remove log (%s)");
    }
    if (cld->name == NULL) {
	failed_title = _("Failed to add log");
	failed_text = _("Could not add log (%s)");
    }
    if (e != NULL) {
	warning_dialog(cld->conc, failed_title, failed_text, e->message);
	g_clear_error(&e);
	if (cld->xml != NULL) {
	    return;
	}
    }
    config_logs_dialog_destroy(cld);
}

static void
config_logs_dialog_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;

    if (arg1 == GTK_RESPONSE_OK) {
	const gchar *name;
	const gchar *file;
	gchar *member;
	gchar *command;
	SpecLog *sl;
	MessageArg *a;
	GError *e = NULL;
	GtkTreeIter iter;

	name = gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cld->xml,
			CONFIG_LOG_DIALOG_NAME)));
	if (name == NULL || strlen(name) == 0) {
	     warning_dialog(cld->conc, _("Cannot add log"),
		     _("No name was given"));
	     return;
	}
	file = gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cld->xml,
			CONFIG_LOG_DIALOG_FILE)));
	if (file == NULL || strlen(file) == 0) {
	     warning_dialog(cld->conc, _("Cannot add log"),
		     _("No file was given"));
	     return;
	}
	sl = g_new0(struct _SpecLog, 1);
	sl->name = g_strdup(name);
	sl->file = g_strdup(file);
	sl->server = get_combobox_text(cld->xml, CONFIG_LOG_DIALOG_SERVER,
	    GTK_TREE_MODEL(cld->conc->all_servers), SERVERS_COLUMN_NAME);
	sl->consoles = NULL;
	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(cld->members),
		    &iter) == TRUE) {
	    do {
		gtk_tree_model_get(GTK_TREE_MODEL(cld->members), &iter,
			COLUMN_MEMBER, &member, -1);
		sl->consoles = g_list_prepend(sl->consoles, member);
	    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(cld->members),
			&iter) == TRUE);
	}
	a = spec_log_message_arg("spec", sl);
	spec_log_free(sl);

	if (cld->name == NULL) {
	    command = CLIENT_MESSAGE_CONFIG_LOG_ADD;
	} else {
	    MessageArg *spec = a;

	    command = CLIENT_MESSAGE_CONFIG_LOG_EDIT;
	    a = message_arg_list("", NULL);
	    a->list = g_list_prepend(a->list,
		    message_arg_string("name", cld->name));
	    a->list = g_list_prepend(a->list, spec);
	}
	if (message_send(cld->conc->msg_q, command, a, config_logs_done,
		    cld, &e) == FALSE) {
	    error_dialog(cld->conc, _("Internal error"),
		    _("Failed to send message (%s)"), e->message);
	    g_clear_error(&e);
	}
	message_arg_free(a);
	return;
    }
    config_logs_dialog_destroy(cld);
}

static void
config_logs_consoles_changed(GtkTreeView *treeview,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;
    GtkTreeIter iter;
    gboolean sensitive = FALSE;

    if (gtk_tree_selection_get_selected(
		gtk_tree_view_get_selection(cld->treeview), NULL, &iter)
	    == TRUE) {
	sensitive = TRUE;
    }
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_REMOVE),
	    "sensitive", sensitive, NULL);
}

static void
config_logs_console_changed(GtkComboBox *combobox,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;
    gchar *selection;
    GtkTreeIter iter;
    gchar *member;
    gboolean sensitive = TRUE;

    selection = get_combobox_text(cld->xml, CONFIG_LOG_DIALOG_CONSOLE,
	    GTK_TREE_MODEL(cld->conc->all_consoles), CONSOLES_COLUMN_NAME);
    if (selection != NULL &&
	    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(cld->members),
		&iter) == TRUE) {
	do {
	    gtk_tree_model_get(GTK_TREE_MODEL(cld->members), &iter,
		    COLUMN_MEMBER, &member, -1);
	    if (strcmp(selection, member) == 0) {
		sensitive = FALSE;
		g_free(member);
		break;
	    }
	    g_free(member);
	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(cld->members),
		    &iter) == TRUE);
    }
    if (selection == NULL) {
	sensitive = FALSE;
    }
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_ADD),
	    "sensitive", sensitive, NULL);
    g_free(selection);
}

static void
config_logs_console_remove(GtkButton *button,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(
		gtk_tree_view_get_selection(cld->treeview), NULL, &iter)
	    == TRUE) {
	gtk_list_store_remove(cld->members, &iter);
    }
    config_logs_consoles_changed(NULL, cld);
    config_logs_console_changed(NULL, cld);
}

static void
config_logs_console_add(GtkButton *button,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;
    gchar *selection;
    GtkTreeIter iter;

    selection = get_combobox_text(cld->xml, CONFIG_LOG_DIALOG_CONSOLE,
	    GTK_TREE_MODEL(cld->conc->all_consoles), CONSOLES_COLUMN_NAME);
    if (selection == NULL) {
	return;
    }
    gtk_list_store_append(cld->members, &iter);
    gtk_list_store_set(cld->members, &iter,
	    COLUMN_MEMBER, selection, -1);
    config_logs_consoles_changed(NULL, cld);
    config_logs_console_changed(NULL, cld);
}

static void
config_logs_dialog(ConfigLogDialog *cld) {
    GtkCellRenderer *text_rend;
    GtkTreeViewColumn *col;

    cld->xml = glade_xml_new(cld->conc->glade_file, CONFIG_LOG_DIALOG, NULL);
    if (cld->name != NULL) {
	gchar *title = g_strdup_printf(_("Edit Log: %s"), cld->name);

	g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG),
		"title", title, NULL);
	g_free(title);
    }
    text_rend = gtk_cell_renderer_text_new();
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_SERVER),
	    "model", cld->conc->all_servers, NULL);
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_SERVER)), text_rend, TRUE);
    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_SERVER)), text_rend, "text",
	    SERVERS_COLUMN_NAME);
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_CONSOLE),
	    "model", cld->conc->all_consoles, NULL);
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_CONSOLE)), text_rend, TRUE);
    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_CONSOLE)), text_rend, "text",
	    CONSOLES_COLUMN_NAME);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(cld->members),
	    COLUMN_MEMBER, GTK_SORT_ASCENDING);
    cld->treeview = GTK_TREE_VIEW(glade_xml_get_widget(cld->xml,
		CONFIG_LOG_DIALOG_CONSOLES));
    g_object_set(cld->treeview,
	    "model", cld->members,
	    "rules-hint", TRUE,
	    "headers-clickable", TRUE,
	    "enable-search", TRUE,
	    "search-column", COLUMN_MEMBER,
	    NULL);
    col = gtk_tree_view_column_new_with_attributes("Console", text_rend,
	    "text", COLUMN_MEMBER, NULL);
    g_object_set(col,
	    "reorderable", TRUE,
	    "sort-indicator", TRUE,
	    NULL);
    gtk_tree_view_append_column(cld->treeview, col);
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_ADD),
	    "sensitive", FALSE, NULL);
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_REMOVE),
	    "sensitive", FALSE, NULL);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_CONSOLE_CHANGED,
	    (GCallback)config_logs_console_changed, cld);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_CONSOLES_CHANGED,
	    (GCallback)config_logs_consoles_changed, cld);
    gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_CONSOLE)), 0);
    gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_SERVER)), 0);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_ADD_CONSOLE,
	    (GCallback)config_logs_console_add, cld);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_REMOVE_CONSOLE,
	    (GCallback)config_logs_console_remove, cld);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_RESPONSE,
	    (GCallback)config_logs_dialog_response, cld);
}

static void
make_list(ConfigLogDialog *cld,
	GList *members) {
    GList *l;
    GtkTreeIter iter;

    cld->members = gtk_list_store_new(COLUMN_LAST, G_TYPE_STRING);
    for (l = members; l != NULL; l = g_list_next(l)) {
	gtk_list_store_append(cld->members, &iter);
	gtk_list_store_set(cld->members, &iter,
		COLUMN_MEMBER, l->data, -1);
    }
}

extern void
config_logs_add(Conc *conc) {
    ConfigLogDialog *cld = g_new0(struct _ConfigLogDialog, 1);

    cld->conc = conc;
    cld->name = NULL;
    make_list(cld, NULL);
    config_logs_dialog(cld);
}

static void
config_logs_edit_data(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    ConfigLogDialog *cld = data;
    GError *e = message_arg_error_check(arg);
    SpecLog *sl;

    if (e != NULL) {
	warning_dialog(cld->conc, _("Failed to edit log"),
		_("Could not edit log (%s)"), e->message);
	g_clear_error(&e);
	config_logs_dialog_destroy(cld);
	return;
    }
    sl = spec_log_new_from_arg(arg);
    g_assert(sl);
    make_list(cld, sl->consoles);
    config_logs_dialog(cld);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_NAME)), sl->name);
    set_combobox_text(cld->xml, CONFIG_LOG_DIALOG_SERVER,
	    GTK_TREE_MODEL(cld->conc->all_servers), SERVERS_COLUMN_NAME,
	    sl->server);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_FILE)), sl->file);
    spec_log_free(sl);
}

extern void
config_logs_edit(Conc *conc,
	const gchar *name) {
    ConfigLogDialog *cld = g_new0(struct _ConfigLogDialog, 1);
    MessageArg *a;
    GError *e = NULL;

    cld->conc = conc;
    cld->name = g_strdup(name);
    cld->xml = NULL;
    cld->members = NULL;
    a = message_arg_string("name", cld->name);
    if (message_send(cld->conc->msg_q, CLIENT_MESSAGE_CONFIG_LOG_SPEC,
		a, config_logs_edit_data, cld, &e) == FALSE) {
	error_dialog(cld->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

extern void
config_logs_remove(Conc *conc,
	const gchar *name) {
    ConfigLogDialog *cld = g_new0(struct _ConfigLogDialog, 1);
    MessageArg *a;
    GError *e = NULL;

    cld->conc = conc;
    cld->name = g_strdup(name);
    cld->xml = NULL;
    cld->members = NULL;
    a = message_arg_string("name", name);
    if (message_send(cld->conc->msg_q, CLIENT_MESSAGE_CONFIG_LOG_REMOVE,
		a, config_logs_done, cld, &e) == FALSE) {
	error_dialog(cld->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

==> conc-2.0.3.tar.gz: conc-2.0.3/client/config-logs.c <==
#include <string.h>
#include "config-logs.h"
#include "spec-log.h"
#include "error.h"
#include "client-message.h"

typedef struct _ConfigLogDialog {
    Conc *conc;
    gchar *name; /* NULL if adding new log */
    GladeXML *xml; /* NULL if removing log */
    GtkTreeView *treeview;
    GtkListStore *members;
} ConfigLogDialog;

enum {
    COLUMN_MEMBER,
    COLUMN_LAST
};

/*
 * Glade widget names:
 */
#define CONFIG_LOG_DIALOG			"config log dialog"
#define CONFIG_LOG_DIALOG_NAME			"cldname"
#define CONFIG_LOG_DIALOG_SERVER		"cldserver"
#define CONFIG_LOG_DIALOG_FILE			"cldfile"
#define CONFIG_LOG_DIALOG_CONSOLES		"cldconsoles"
#define CONFIG_LOG_DIALOG_CONSOLE		"cldconsole"
#define CONFIG_LOG_DIALOG_ADD			"cldadd"
#define CONFIG_LOG_DIALOG_REMOVE		"cldremove"

/*
 * Glade handler names:
 */
#define CONFIG_LOG_DIALOG_RESPONSE		"response"
#define CONFIG_LOG_DIALOG_CONSOLES_CHANGED	"consoles changed"
#define CONFIG_LOG_DIALOG_CONSOLE_CHANGED	"console changed"
#define CONFIG_LOG_DIALOG_ADD_CONSOLE		"console add"
#define CONFIG_LOG_DIALOG_REMOVE_CONSOLE	"console remove"

static void
config_logs_dialog_destroy(ConfigLogDialog *cld) {
    if (cld->xml) {
	gtk_widget_destroy(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG));
    }
    if (cld->name) {
	g_free(cld->name);
    }
    if (cld->members) {
	g_object_unref(cld->members);
    }
    g_free(cld);
}

static void
config_logs_done(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    ConfigLogDialog *cld = data;
    GError *e = message_arg_error_check(arg);
    gchar *failed_title = _("Failed to update log");
    gchar *failed_text = _("Could not modify log (%s)");

    if (cld->xml == NULL) {
	failed_title = _("Failed to remove log");
	failed_text = _("Could not remove log (%s)");
    }
    if (cld->name == NULL) {
	failed_title = _("Failed to add log");
	failed_text = _("Could not add log (%s)");
    }
    if (e != NULL) {
	warning_dialog(cld->conc, failed_title, failed_text, e->message);
	g_clear_error(&e);
	if (cld->xml != NULL) {
	    return;
	}
    }
    config_logs_dialog_destroy(cld);
}

static void
config_logs_dialog_response(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;

    if (arg1 == GTK_RESPONSE_OK) {
	const gchar *name;
	const gchar *file;
	gchar *member;
	gchar *command;
	SpecLog *sl;
	MessageArg *a;
	GError *e = NULL;
	GtkTreeIter iter;

	name = gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cld->xml,
			CONFIG_LOG_DIALOG_NAME)));
	if (name == NULL || strlen(name) == 0) {
	     warning_dialog(cld->conc, _("Cannot add log"),
		     _("No name was given"));
	     return;
	}
	file = gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(cld->xml,
			CONFIG_LOG_DIALOG_FILE)));
	if (file == NULL || strlen(file) == 0) {
	     warning_dialog(cld->conc, _("Cannot add log"),
		     _("No file was given"));
	     return;
	}
	sl = g_new0(struct _SpecLog, 1);
	sl->name = g_strdup(name);
	sl->file = g_strdup(file);
	sl->server = get_combobox_text(cld->xml, CONFIG_LOG_DIALOG_SERVER,
	    GTK_TREE_MODEL(cld->conc->all_servers), SERVERS_COLUMN_NAME);
	sl->consoles = NULL;
	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(cld->members),
		    &iter) == TRUE) {
	    do {
		gtk_tree_model_get(GTK_TREE_MODEL(cld->members), &iter,
			COLUMN_MEMBER, &member, -1);
		sl->consoles = g_list_prepend(sl->consoles, member);
	    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(cld->members),
			&iter) == TRUE);
	}
	a = spec_log_message_arg("spec", sl);
	spec_log_free(sl);

	if (cld->name == NULL) {
	    command = CLIENT_MESSAGE_CONFIG_LOG_ADD;
	} else {
	    MessageArg *spec = a;

	    command = CLIENT_MESSAGE_CONFIG_LOG_EDIT;
	    a = message_arg_list("", NULL);
	    a->list = g_list_prepend(a->list,
		    message_arg_string("name", cld->name));
	    a->list = g_list_prepend(a->list, spec);
	}
	if (message_send(cld->conc->msg_q, command, a, config_logs_done,
		    cld, &e) == FALSE) {
	    error_dialog(cld->conc, _("Internal error"),
		    _("Failed to send message (%s)"), e->message);
	    g_clear_error(&e);
	}
	message_arg_free(a);
	return;
    }
    config_logs_dialog_destroy(cld);
}

static void
config_logs_consoles_changed(GtkTreeView *treeview,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;
    GtkTreeIter iter;
    gboolean sensitive = FALSE;

    if (gtk_tree_selection_get_selected(
		gtk_tree_view_get_selection(cld->treeview), NULL, &iter)
	    == TRUE) {
	sensitive = TRUE;
    }
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_REMOVE),
	    "sensitive", sensitive, NULL);
}

static void
config_logs_console_changed(GtkComboBox *combobox,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;
    gchar *selection;
    GtkTreeIter iter;
    gchar *member;
    gboolean sensitive = TRUE;

    selection = get_combobox_text(cld->xml, CONFIG_LOG_DIALOG_CONSOLE,
	    GTK_TREE_MODEL(cld->conc->all_consoles), CONSOLES_COLUMN_NAME);
    if (selection != NULL &&
	    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(cld->members),
		&iter) == TRUE) {
	do {
	    gtk_tree_model_get(GTK_TREE_MODEL(cld->members), &iter,
		    COLUMN_MEMBER, &member, -1);
	    if (strcmp(selection, member) == 0) {
		sensitive = FALSE;
		g_free(member);
		break;
	    }
	    g_free(member);
	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(cld->members),
		    &iter) == TRUE);
    }
    if (selection == NULL) {
	sensitive = FALSE;
    }
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_ADD),
	    "sensitive", sensitive, NULL);
    g_free(selection);
}

static void
config_logs_console_remove(GtkButton *button,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(
		gtk_tree_view_get_selection(cld->treeview), NULL, &iter)
	    == TRUE) {
	gtk_list_store_remove(cld->members, &iter);
    }
    config_logs_consoles_changed(NULL, cld);
    config_logs_console_changed(NULL, cld);
}

static void
config_logs_console_add(GtkButton *button,
	gpointer user_data) {
    ConfigLogDialog *cld = user_data;
    gchar *selection;
    GtkTreeIter iter;

    selection = get_combobox_text(cld->xml, CONFIG_LOG_DIALOG_CONSOLE,
	    GTK_TREE_MODEL(cld->conc->all_consoles), CONSOLES_COLUMN_NAME);
    if (selection == NULL) {
	return;
    }
    gtk_list_store_append(cld->members, &iter);
    gtk_list_store_set(cld->members, &iter,
	    COLUMN_MEMBER, selection, -1);
    config_logs_consoles_changed(NULL, cld);
    config_logs_console_changed(NULL, cld);
}

static void
config_logs_dialog(ConfigLogDialog *cld) {
    GtkCellRenderer *text_rend;
    GtkTreeViewColumn *col;

    cld->xml = glade_xml_new(cld->conc->glade_file, CONFIG_LOG_DIALOG, NULL);
    if (cld->name != NULL) {
	gchar *title = g_strdup_printf(_("Edit Log: %s"), cld->name);

	g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG),
		"title", title, NULL);
	g_free(title);
    }
    text_rend = gtk_cell_renderer_text_new();
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_SERVER),
	    "model", cld->conc->all_servers, NULL);
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_SERVER)), text_rend, TRUE);
    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_SERVER)), text_rend, "text",
	    SERVERS_COLUMN_NAME);
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_CONSOLE),
	    "model", cld->conc->all_consoles, NULL);
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_CONSOLE)), text_rend, TRUE);
    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_CONSOLE)), text_rend, "text",
	    CONSOLES_COLUMN_NAME);
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(cld->members),
	    COLUMN_MEMBER, GTK_SORT_ASCENDING);
    cld->treeview = GTK_TREE_VIEW(glade_xml_get_widget(cld->xml,
		CONFIG_LOG_DIALOG_CONSOLES));
    g_object_set(cld->treeview,
	    "model", cld->members,
	    "rules-hint", TRUE,
	    "headers-clickable", TRUE,
	    "enable-search", TRUE,
	    "search-column", COLUMN_MEMBER,
	    NULL);
    col = gtk_tree_view_column_new_with_attributes("Console", text_rend,
	    "text", COLUMN_MEMBER, NULL);
    g_object_set(col,
	    "reorderable", TRUE,
	    "sort-indicator", TRUE,
	    NULL);
    gtk_tree_view_append_column(cld->treeview, col);
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_ADD),
	    "sensitive", FALSE, NULL);
    g_object_set(glade_xml_get_widget(cld->xml, CONFIG_LOG_DIALOG_REMOVE),
	    "sensitive", FALSE, NULL);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_CONSOLE_CHANGED,
	    (GCallback)config_logs_console_changed, cld);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_CONSOLES_CHANGED,
	    (GCallback)config_logs_consoles_changed, cld);
    gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_CONSOLE)), 0);
    gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_SERVER)), 0);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_ADD_CONSOLE,
	    (GCallback)config_logs_console_add, cld);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_REMOVE_CONSOLE,
	    (GCallback)config_logs_console_remove, cld);
    glade_xml_signal_connect_data(cld->xml,
	    CONFIG_LOG_DIALOG_RESPONSE,
	    (GCallback)config_logs_dialog_response, cld);
}

static void
make_list(ConfigLogDialog *cld,
	GList *members) {
    GList *l;
    GtkTreeIter iter;

    cld->members = gtk_list_store_new(COLUMN_LAST, G_TYPE_STRING);
    for (l = members; l != NULL; l = g_list_next(l)) {
	gtk_list_store_append(cld->members, &iter);
	gtk_list_store_set(cld->members, &iter,
		COLUMN_MEMBER, l->data, -1);
    }
}

extern void
config_logs_add(Conc *conc) {
    ConfigLogDialog *cld = g_new0(struct _ConfigLogDialog, 1);

    cld->conc = conc;
    cld->name = NULL;
    make_list(cld, NULL);
    config_logs_dialog(cld);
}

static void
config_logs_edit_data(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    ConfigLogDialog *cld = data;
    GError *e = message_arg_error_check(arg);
    SpecLog *sl;

    if (e != NULL) {
	warning_dialog(cld->conc, _("Failed to edit log"),
		_("Could not edit log (%s)"), e->message);
	g_clear_error(&e);
	config_logs_dialog_destroy(cld);
	return;
    }
    sl = spec_log_new_from_arg(arg);
    g_assert(sl);
    make_list(cld, sl->consoles);
    config_logs_dialog(cld);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_NAME)), sl->name);
    set_combobox_text(cld->xml, CONFIG_LOG_DIALOG_SERVER,
	    GTK_TREE_MODEL(cld->conc->all_servers), SERVERS_COLUMN_NAME,
	    sl->server);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(cld->xml,
		    CONFIG_LOG_DIALOG_FILE)), sl->file);
    spec_log_free(sl);
}

extern void
config_logs_edit(Conc *conc,
	const gchar *name) {
    ConfigLogDialog *cld = g_new0(struct _ConfigLogDialog, 1);
    MessageArg *a;
    GError *e = NULL;

    cld->conc = conc;
    cld->name = g_strdup(name);
    cld->xml = NULL;
    cld->members = NULL;
    a = message_arg_string("name", cld->name);
    if (message_send(cld->conc->msg_q, CLIENT_MESSAGE_CONFIG_LOG_SPEC,
		a, config_logs_edit_data, cld, &e) == FALSE) {
	error_dialog(cld->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

extern void
config_logs_remove(Conc *conc,
	const gchar *name) {
    ConfigLogDialog *cld = g_new0(struct _ConfigLogDialog, 1);
    MessageArg *a;
    GError *e = NULL;

    cld->conc = conc;
    cld->name = g_strdup(name);
    cld->xml = NULL;
    cld->members = NULL;
    a = message_arg_string("name", name);
    if (message_send(cld->conc->msg_q, CLIENT_MESSAGE_CONFIG_LOG_REMOVE,
		a, config_logs_done, cld, &e) == FALSE) {
	error_dialog(cld->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}
